@@ -12,11 +12,14 @@
 #include <QDebug>
 #include <QThread>
 
-Database* databaseInstance = 0;
+Database::Database* databaseInstance = 0;
 
 Database::Database() :
     QObject(),
-    m_thread(new QThread)
+    m_thread(new QThread),
+    m_database(QSqlDatabase::addDatabase("QSQLITE")),
+    m_busy(false),
+    m_cancelled(false)
 {
     if (!databaseInstance) {
         databaseInstance = this;
@@ -24,12 +27,17 @@ Database::Database() :
 
     this->moveToThread(m_thread);
     m_thread->start();
-
+#if (defined (Q_WS_MAEMO_5)) || (defined (MEEGO_EDITION_HARMATTAN))
     QDir dir;
     dir.mkpath("/home/user/MusiKloud/");
-    m_database = QSqlDatabase::addDatabase("QSQLITE");
     m_database.setDatabaseName("/home/user/MusiKloud/MusiKloud.sqlite");
-
+#elif (defined (SYMBIAN_OS))
+    m_database.setDatabaseName("MusiKloud.sqlite");
+#else
+    QDir dir(QDesktopServices::storageLocation(QDesktopServices::HomeLocation) + "/MusiKloud/");
+    dir.mkpath(dir.path());
+    m_database.setDatabaseName(dir.path() + "/MusiKloud.sqlite");
+#endif
     this->initialize();
 }
 
@@ -52,8 +60,8 @@ void Database::initialize() {
         m_database.exec("CREATE TABLE IF NOT EXISTS playlistDownloads (id TEXT UNIQUE, artist TEXT, title TEXT, description TEXT, coverUrl TEXT)");
         m_database.exec("CREATE TABLE IF NOT EXISTS lastfmAccount (name TEXT UNIQUE, user TEXT, token TEXT)");
         m_database.exec("CREATE TABLE IF NOT EXISTS soundcloudAccount (name TEXT UNIQUE, user TEXT, token TEXT)");
-        m_database.exec("ADD COLUMN trackDownloads fileSize TEXT");
-        m_database.exec("ADD COLUMN playlistDownloads trackCount INTEGER");
+        m_database.exec("ALTER TABLE trackDownloads ADD COLUMN fileSize TEXT");
+        m_database.exec("ALTER TABLE playlistDownloads ADD COLUMN trackCount INTEGER");
         m_database.close();
     }
     else {
@@ -264,17 +272,18 @@ bool Database::storeDownload(QSharedPointer<TransferItem> transfer) {
         m_database.open();
     }
 
-    int i = 0;
-    bool success = false;
+    bool success = this->storeTrackDownload(transfer.data()->track());
 
     switch (transfer.data()->transferType()) {
     case Transfers::PlaylistDownload:
         success = this->storePlaylistDownload(transfer.data()->playlist());
         break;
     default:
-        return this->storeTrackDownload(transfer.data()->track());
+        return success;
     }
 
+    int i = 0;
+
     while ((success) && (i < transfer.data()->playlistTracks().size())) {
         success = this->storeTrackDownload(transfer.data()->playlistTracks().at(i));
         i++;
@@ -380,14 +389,22 @@ QList< QSharedPointer<TransferItem> > Database::getStoredDownloads() {
     QList< QSharedPointer<TrackItem> > tracks = this->getStoredTrackDownloads();
     QList< QSharedPointer<PlaylistItem> > playlists = this->getStoredPlaylistDownloads();
 
+    qDebug() << "Found " + QString::number(tracks.size()) + " track downloads";
+    qDebug() << "Found " + QString::number(playlists.size()) + " playlist downloads";
+
     while (!playlists.isEmpty()) {
         QSharedPointer<PlaylistItem> playlist = playlists.takeFirst();
         QList< QSharedPointer<TrackItem> > playlistTracks;
 
-        for (int i = 0; i < tracks.size(); i++) {
+        int i = 0;
+
+        while (i < tracks.size()) {
             if (tracks.at(i).data()->playlistId() == playlist.data()->id()) {
                 playlistTracks.append(tracks.takeAt(i));
             }
+            else {
+                i++;
+            }
         }
 
         transfers.append(QSharedPointer<TransferItem>(new TransferItem(playlist, playlistTracks)));
@@ -457,6 +474,7 @@ QList < QSharedPointer<PlaylistItem> > Database::getStoredPlaylistDownloads() {
             playlist.data()->setTitle(query.value(2).toString());
             playlist.data()->setDescription(query.value(3).toString());
             playlist.data()->setThumbnailUrl(query.value(4).toString());
+            playlist.data()->setTrackCount(query.value(5).toInt());
             playlists.append(playlist);
         }
     }
@@ -620,7 +638,7 @@ bool Database::addTrack(QSharedPointer<TrackItem> track) {
     bool success = false;
     QSqlQuery query;
     query.prepare("INSERT OR IGNORE INTO tracks VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
-    query.addBindValue(track.data()->url());
+    query.addBindValue(track.data()->url().toLocalFile());
     query.addBindValue(track.data()->id());
     query.addBindValue(track.data()->artist());
     query.addBindValue(track.data()->title());
@@ -652,7 +670,7 @@ bool Database::updateTrack(const QUrl &url, const QString &key, const QVariant &
         QSqlQuery query;
         query.prepare(QString("UPDATE tracks SET %1 = ? WHERE filePath = ?").arg(key));
         query.addBindValue(value);
-        query.addBindValue(url);
+        query.addBindValue(url.toLocalFile());
         bool success = query.exec();
         m_database.close();
 
@@ -674,7 +692,7 @@ bool Database::deleteTrack(QSharedPointer<TrackItem> track, bool deleteFromFS, b
     bool success = false;
     QSqlQuery query;
     query.prepare("DELETE FROM tracks WHERE filePath = ?");
-    query.addBindValue(track.data()->url());
+    query.addBindValue(track.data()->url().toLocalFile());
     success = query.exec();
     m_database.close();
 
@@ -734,7 +752,8 @@ QList< QSharedPointer<TrackItem> > Database::getTracks(const QString &playlistId
     if (record.count() > 0) {
         while (query.next()) {
             QSharedPointer<TrackItem> track = QSharedPointer<TrackItem>(new TrackItem);
-            track.data()->setUrl(query.value(0).toUrl());
+            track.data()->setUrl(QUrl::fromLocalFile(query.value(0).toString().remove("file://")));
+            track.data()->setStreamUrl(track.data()->url());
             track.data()->setId(query.value(1).toString());
             track.data()->setArtist(query.value(2).toString());
             track.data()->setTitle(query.value(3).toString());
@@ -747,7 +766,7 @@ QList< QSharedPointer<TrackItem> > Database::getTracks(const QString &playlistId
             track.data()->setPlaylistId(query.value(9).toString());
             track.data()->setTrackNumber(query.value(10).toInt());
             track.data()->setPlayCount(query.value(11).toInt());
-            track.data()->setLastPlayed(Utils::dateFromMSecs(query.value(12).toLongLong()));
+            track.data()->setLastPlayed(query.value(12).toLongLong());
             track.data()->setSize(query.value(13).toLongLong());
             track.data()->setFormat(query.value(14).toString());
             track.data()->setDate(Utils::dateFromMSecs(query.value(15).toLongLong()));
@@ -772,7 +791,7 @@ QList< QSharedPointer<PlaylistItem> > Database::getPlaylists() {
     QSqlQuery query;
     QSqlRecord record;
     QList< QSharedPointer<PlaylistItem> > playlists;
-    query.prepare("SELECT * FROM tracks ORDER BY lastModified DESC");
+    query.prepare("SELECT * FROM playlists ORDER BY lastModified DESC");
     query.exec();
     record = query.record();
 
