@@ -17,7 +17,7 @@ const QString CLIENT_ID("aada80a82c91fb42d0bb1cb321b59034");
 const QString CLIENT_SECRET("dcc9c80ce05614d4988cebb82ddde152");
 const QString REDIRECT_URI("http://www.musikloud.co.uk");
 
-SoundCloud* soundCloudInstance = 0;
+SoundCloud::SoundCloud* soundCloudInstance = 0;
 
 using namespace QtJson;
 
@@ -62,8 +62,8 @@ void SoundCloud::cancelCurrentOperation() {
     this->disconnect(this, SIGNAL(gotResourceId(QString,Queries::QueryType)), 0, 0);
 #ifdef QML_USER_INTERFACE
     this->disconnect(this, SIGNAL(gotTrack(TrackItem*)), 0, 0);
-    this->disconnect(this, SIGNAL(gotPlaylist(PlaylistItem*)), 0, 0)
-        #else
+    this->disconnect(this, SIGNAL(gotPlaylist(PlaylistItem*)), 0, 0);
+#else
     this->disconnect(this, SIGNAL(gotTrack(QSharedPointer<TrackItem>)), 0, 0);
     this->disconnect(this, SIGNAL(gotPlaylist(QSharedPointer<PlaylistItem>)), 0, 0);
 #endif
@@ -82,16 +82,12 @@ QUrl SoundCloud::authUrl() {
 }
 
 QNetworkReply* SoundCloud::createReply(QUrl url, int offset) {
-    if (url.scheme() == "https") {
-        if (this->userSignedIn()) {
-            url.addQueryItem("oauth_token", this->accessToken());
-        }
-        else {
-            url.setScheme("http");
-            url.addQueryItem("client_id", CLIENT_ID);
-        }
+    if (this->userSignedIn()) {
+        url.setScheme("https");
+        url.addQueryItem("oauth_token", this->accessToken());
     }
     else {
+        url.setScheme("http");
         url.addQueryItem("client_id", CLIENT_ID);
     }
 
@@ -116,7 +112,6 @@ QNetworkReply* SoundCloud::createSearchReply(int queryType, const QString &query
     switch (queryType) {
     case Queries::Tracks:
         url = TRACKS_BASE_FEED;
-        url.addQueryItem("filter", "streamable");
         break;
     case Queries::Playlists:
         url = PLAYLISTS_BASE_FEED;
@@ -167,8 +162,14 @@ QUrl SoundCloud::getStreamUrl(QUrl url) {
 }
 
 QUrl SoundCloud::getDownloadUrl(QUrl url) {
-    if (this->userSignedIn()) {
-        url.addQueryItem("oauth_token", this->accessToken());
+    if (url.scheme() == "https") {
+        if (this->userSignedIn()) {
+            url.addQueryItem("oauth_token", this->accessToken());
+        }
+        else {
+            url.setScheme("http");
+            url.addQueryItem("client_id", CLIENT_ID);
+        }
     }
     else {
         url.addQueryItem("client_id", CLIENT_ID);
@@ -369,15 +370,6 @@ void SoundCloud::getUserProfile(const QString &id) {
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkUser()));
 }
 
-void SoundCloud::getCurrentUserProfile() {
-    QUrl url("https://api.soundcloud.com/me.json");
-    url.addQueryItem("oauth_token", this->accessToken());
-    QNetworkRequest request(url);
-    request.setRawHeader("User-Agent", QString("MusiKloud/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
-    QNetworkReply *reply = this->networkAccessManager()->get(request);
-    this->connect(reply, SIGNAL(finished()), this, SLOT(checkCurrentUser()));
-}
-
 void SoundCloud::checkUser() {
     QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
@@ -407,15 +399,29 @@ void SoundCloud::checkUser() {
             this->isUserFollowed(user->id());
         }
 #ifdef QML_USER_INTERFACE
+        QDeclarativeEngine::setObjectOwnership(user, QDeclarativeEngine::JavaScriptOwnership);
         emit gotUser(user);
 #else
         emit gotUser(QSharedPointer<UserItem>(user));
 #endif
     }
-
+#ifdef QML_USER_INTERFACE
+    this->disconnect(this, SIGNAL(gotUser(UserItem*)), 0, 0);
+#else
+    this->disconnect(this, SIGNAL(gotUser(QSharedPointer<UserItem>)), 0, 0);
+#endif
     reply->deleteLater();
 }
 
+void SoundCloud::getCurrentUserProfile() {
+    QUrl url("https://api.soundcloud.com/me.json");
+    url.addQueryItem("oauth_token", this->accessToken());
+    QNetworkRequest request(url);
+    request.setRawHeader("User-Agent", QString("MusiKloud/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
+    QNetworkReply *reply = this->networkAccessManager()->get(request);
+    this->connect(reply, SIGNAL(finished()), this, SLOT(checkCurrentUser()));
+}
+
 void SoundCloud::checkCurrentUser() {
     QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
@@ -450,6 +456,7 @@ void SoundCloud::checkCurrentUser() {
         emit gotUsername(user->username());
     }
 #ifdef QML_USER_INTERFACE
+    QDeclarativeEngine::setObjectOwnership(user, QDeclarativeEngine::JavaScriptOwnership);
     emit gotUser(user);
 #else
     emit gotUser(QSharedPointer<UserItem>(user));
@@ -613,6 +620,8 @@ void SoundCloud::setAccessToken(const QString &token) {
 }
 
 void SoundCloud::postRequest(const QUrl &url, const QByteArray &data) {
+    qDebug() << url;
+    qDebug() << data;
     QNetworkRequest request(url);
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
     request.setRawHeader("User-Agent", QString("MusiKloud/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
@@ -622,6 +631,7 @@ void SoundCloud::postRequest(const QUrl &url, const QByteArray &data) {
 }
 
 void SoundCloud::putRequest(const QUrl &url) {
+    qDebug() << url;
     QNetworkRequest request(url);
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
     request.setRawHeader("User-Agent", QString("MusiKloud/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
@@ -631,6 +641,7 @@ void SoundCloud::putRequest(const QUrl &url) {
 }
 
 void SoundCloud::deleteRequest(const QUrl &url) {
+    qDebug() << url;
     QNetworkRequest request(url);
     QNetworkReply *reply = this->networkAccessManager()->deleteResource(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(postFinished()));
@@ -653,8 +664,6 @@ void SoundCloud::postFinished() {
         emit postFailed(reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toString());
     }
 
-    this->disconnect(this, SIGNAL(postSuccessful(QString)), 0, 0);
-    this->disconnect(this, SIGNAL(postFailed(QString)), 0, 0);
     reply->deleteLater();
 }
 
@@ -681,13 +690,15 @@ void SoundCloud::onCommentAdded(const QString &response) {
     else {
         emit error(tr("Comment response invalid"));
     }
+
+    this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onCommentAdded(QString)));
 }
 
 void SoundCloud::addToFavourites(const QStringList &ids) {
     if (!ids.isEmpty()) {
         m_actionIds = ids;
         m_actionsProcessed = 0;
-        this->setBusy(true, tr("Adding track(s) to favourites"));
+        this->setBusy(true, tr("Adding track(s) to favourites"), m_actionIds.size());
         this->addToFavourites();
     }
 }
@@ -715,6 +726,9 @@ void SoundCloud::onAddedToFavourites() {
     else {
         this->setBusy(false);
         emit alert(tr("Track(s) added to favourites"));
+
+        this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onAddedToFavourites()));
+        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onTrackActionError(QString)));
     }
 }
 
@@ -722,7 +736,7 @@ void SoundCloud::deleteFromFavourites(const QStringList &ids) {
     if (!ids.isEmpty()) {
         m_actionIds = ids;
         m_actionsProcessed = 0;
-        this->setBusy(true, tr("Deleting track(s) from favourites"));
+        this->setBusy(true, tr("Deleting track(s) from favourites"), m_actionIds.size());
         this->deleteFromFavourites();
     }
 }
@@ -750,6 +764,9 @@ void SoundCloud::onDeletedFromFavourites() {
     else {
         this->setBusy(false);
         emit alert(tr("Track(s) deleted from favourites"));
+
+        this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onDeletedFromFavourites()));
+        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onTrackActionError(QString)));
     }
 }
 
@@ -757,6 +774,8 @@ void SoundCloud::onTrackActionError(const QString &errorString) {
     this->setBusy(false);
     emit error(errorString);
     m_actionIds.clear();
+    this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onTrackActionError(QString)));
+    this->disconnect(this, SIGNAL(postSuccessful(QString)), this, 0);
 }
 
 void SoundCloud::followUser(const QString &id) {
@@ -771,6 +790,8 @@ void SoundCloud::followUser(const QString &id) {
 void SoundCloud::onUserFollowed() {
     emit followingChanged(m_actionId, true);
     emit alert(tr("You have followed this user"));
+    this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onUserFollowed()));
+    this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onUserActionError(QString)));
 }
 
 void SoundCloud::unfollowUser(const QString &id) {
@@ -785,10 +806,14 @@ void SoundCloud::unfollowUser(const QString &id) {
 void SoundCloud::onUserUnfollowed() {
     emit followingChanged(m_actionId, false);
     emit alert(tr("You no longer follow this user"));
+    this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onUserUnfollowed()));
+    this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onUserActionError(QString)));
 }
 
 void SoundCloud::onUserActionError(const QString &errorString) {
     emit error(errorString);
+    this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onUserActionError(QString)));
+    this->disconnect(this, SIGNAL(postSuccessful(QString)), this, 0);
 }
 
 void SoundCloud::updateProfile(const QVariantMap &profile) {
@@ -815,6 +840,9 @@ void SoundCloud::onProfileUpdated(const QString &response) {
     else {
         emit error(tr("Cannot obtain new profile"));
     }
+
+    this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onProfileUpdated(QString)));
+    this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onUserActionError(QString)));
 }
 
 void SoundCloud::shareTrack(const QString &id, const QStringList &connections, const QString &message) {
@@ -830,4 +858,5 @@ void SoundCloud::shareTrack(const QString &id, const QStringList &connections, c
 
 void SoundCloud::onTrackShared() {
     emit alert(tr("Track shared to your connections"));
+    this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onTrackShared()));
 }
