@@ -1,7 +1,7 @@
 #include "windowstack.h"
 #include <QWidget>
 
-WindowStack* stackInstance = 0;
+WindowStack::WindowStack* stackInstance = 0;
 
 WindowStack::WindowStack(QObject *parent) :
     QObject(parent)
